#include <stdio.h>
#include <stdlib.h>

#include "lca.h"
#include "rmq.h"

/**
 * Build a random, unbalanced tree with n nodes
 */
static node_t *random_tree( unsigned int n )
{
  unsigned int k;
  node_t *root = NULL;

  /* If there's something to create, do it */
  if ( n > 0 )
  {
    root = (node_t *) malloc( sizeof( node_t ));

    /* Divide the remaining nodes between the two children */
    k = rand() % n;

    root->left  = random_tree( k );
    root->right = random_tree( n - k - 1 );
  }
  
  return root;
}

/**
 * Label a tree by in-order traversal
 */
static unsigned int label_tree( node_t *root, unsigned int label )
{
  if ( root == NULL )
    return label;

  root->value = label++;
  
  label = label_tree( root->left,  label );
  label = label_tree( root->right, label );

  return label;
}

static void print_tree( node_t *root, unsigned int indent )
{
  unsigned int i;

  if ( root == NULL )
    return;

  print_tree( root->right, indent + 1 );
  
  for ( i = 0; i < indent; i++ )
    printf( "  " );
  printf( "%d\n", root->value );

  print_tree( root->left, indent + 1 );
}

static void print_cartesian_tree( node_t *root, unsigned int *a, unsigned int indent )
{ 
  unsigned int i;

  if ( root == NULL )
    return;

  print_cartesian_tree( root->right, a, indent + 1 );
  
  for ( i = 0; i < indent; i++ )
    printf( "  " );
  printf( "[%d:%d]\n", root->value, a[root->value] );

  print_cartesian_tree( root->left, a, indent + 1 );
}

int main(int argc, char *argv[])
{
  /*
  node_t *root   = random_tree( 10 );
  unsigned int n = label_tree( root, 0 );

  lca_init( root, n );
  print_tree( root, 0 );
  */

  unsigned int *a = (unsigned int *) malloc( sizeof( unsigned int ) * 20 );
  unsigned int i, j, m, n, r, s, t, err_cnt = 0;
  quadratic_rmq_t *naive;
  direct_rmq_t *direct;
  sparse_rmq_t *sparse;
  node_t *cart;
  rmq_t *rmq;

  n = 20;

  /* Create a random data set */
  a[0] = m = n;
  for ( i = 1; i < n; i++)
  {
    a[i] = a[i-1] + ( 2 * ( rand() % 2 )) - 1;
    if ( a[i] < m )
      m = a[i];
  }
  for ( i = 0; i < n; i++ )
  {
    a[i] -= m;
    printf("%d ", a[i]);
  }
  printf("\n");

  direct = direct_rmq_init( a, n ); 
  naive  = quadratic_rmq_init( a, n );
  sparse = sparse_rmq_init( a, n );
  rmq    = rmq_init( a, n );
  i   = 0;
  j   = n-1;

  printf( "            \tRMQ\tQUAD\tSPRS\tDIRECT\n" );
  for ( i = 0; i < n; i++ )
    for ( j = i; j < n; j++ )
      printf( "RMQ[%d:%d] =\t%d\t%d\t%d\t%d\n", i, j, rmq_exec( rmq, i, j ), a[quadratic_rmq_exec( naive, i, j )], a[sparse_rmq_exec( sparse, i, j )], a[direct_rmq_exec( direct, i, j)]);
  return 0;
}
