/**
 *
 * Miscellaneous functions
 *
 */

/**
 * Swap two unsigned integers
 */
void uint_swap( unsigned int *a, unsigned int *b )
{
  unsigned int c = *a; *a = *b; *b = c;
}

/**
 * Return the index which contains the larger value
 */
unsigned int uint_argmax( unsigned int *a, unsigned int i, unsigned int j )
{
  return ( a[i] >= a[j] ) ? i : j;
}

unsigned int uint_argmin( unsigned int *a, unsigned int i, unsigned int j )
{
  return ( a[i] <= a[j] ) ? i : j;
}

/**
 * Take the max/min of two unsigned integers
 */
unsigned int uint_max( unsigned int a, unsigned int b )
{
  return ( a > b ) ? a : b; 
}

unsigned int uint_min( unsigned int a, unsigned int b )
{
  return ( a < b ) ? a : b; 
}

/**
 * Compute the log-base-2 of an integer on O(log(n)) operations
 */
unsigned int log_base_2( unsigned int v )
{
  const unsigned int b[] = {0x2, 0xC, 0xF0, 0xFF00, 0xFFFF0000};
  const unsigned int S[] = {1, 2, 4, 8, 16};
  register unsigned int c = 0; /* result of log2(v) will go here */
  
  if ( v & b[4] ) { v >>= S[4]; c |= S[4]; }
  if ( v & b[3] ) { v >>= S[3]; c |= S[3]; }
  if ( v & b[2] ) { v >>= S[2]; c |= S[2]; }
  if ( v & b[1] ) { v >>= S[1]; c |= S[1]; }
  if ( v & b[0] ) { v >>= S[0]; c |= S[0]; }

  return c;
}

/**
 * Allocate an upper-diagonal matrix.  The matrix is created such that is
 * can be accessed via standard indexing, e.g. a[i][j], but with the
 * restriction that j >= i.
 */
unsigned int **malloc_upper_matrix( unsigned int n )
{  
  unsigned int i, total, size, *base, **arr;
  
  /* Compute how much space we need for an upper triangular matrix */
  total = ( n * ( n + 1 )) / 2;
  size  = n * sizeof( unsigned int * ) + total * sizeof( unsigned int ); 

  /* Allocate all the memory in one shot */
  arr = (unsigned int **) malloc( size );

  /* Fill in the pointers to each row */
  base = (unsigned int *) (arr + n);
  for ( i = 0; i < n; i++ )
  {
    arr[i] = base;
    base  += ( n - i - 1 );
  }

  return arr;
}

/**
 * Allocate a standard, 2d matrix.
 */
unsigned int **malloc_unsigned_matrix( unsigned int m, unsigned int n )
{
  unsigned int i, size, *base, **arr;
  
  /* Compute how much space we need for a 2D array */
  size = m * ( sizeof( unsigned int *) + sizeof( unsigned int ) * n );

  /* Allocate all the memory in one shot */
  arr = (unsigned int **) malloc( size );

  /* Fill in the pointers to each row */
  base = (unsigned int *) (arr + m);
  for ( i = 0; i < m; i++ )
  {
    arr[i] = base;
    base  += n;
  }

  return arr;
}
